package com.jz.dm.service.request;

import com.alibaba.fastjson.JSONObject;
import com.jz.common.utils.HttpsUtils;
import com.jz.common.utils.RedisUtils;
import com.jz.dm.common.enums.GatewayResultCode;
import com.jz.dm.common.enums.apiInterface.ApiInfoOutTypeEnum;
import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.util.OpenApiRequest;
import com.jz.dm.common.util.OpenApiResponse;
import com.jz.dm.gateway.OpenApiService;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.service.ApiInterfaceService;
import com.jz.dm.web.annotation.AccessLimit;
import com.jz.dm.web.annotation.ApiLogAspect;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Map;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.request
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiQueryService
 * @DATE: 2020-12-23/18:10
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("apiQueryService")
@Slf4j
public class ApiQueryService implements OpenApiService {
    @Override
    public String getOpenApiMethod() {
        return "query";
    }

    @Override
    public String getOpenApiVersion() {
        return "v1.0.0";
    }

    @Autowired
    private ApiInterfaceService apiInterfaceService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private HttpsUtils httpsUtils;

    @Override
    @ApiLogAspect(description = "API请求日志")
    @AccessLimit(limit = 10000, sec = 1)
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public void doService(OpenApiRequest request, OpenApiResponse response) {
        JSONObject parameter = JSONObject.parseObject(request.getOpenApiParams());
        String selectType = parameter.getString("selectType");
        String reqParams = parameter.getString("reqParams");
        Map paramMap =null;
        if (StringUtils.isNotBlank(reqParams)) {
            paramMap = (Map) JSONObject.parseObject(reqParams);
        }
        String apiInterface = redisUtils.get(request.getAppKey());
        if (StringUtils.isNotBlank(selectType)) {
            try {
                switch (selectType) {
                    case "10001": //实时接入查询
                        break;
                    case "10003": //标签查询
                        break;
                    case "10002"://大数据查询
                        if (StringUtils.isNotBlank(apiInterface)){//redis中存在
                            JSONObject jsonBigData = JSONObject.parseObject(apiInterface);
                            String targetUrl = jsonBigData.getString("targetUrl");
                            rangRequestTarget(ApiInfoOutTypeEnum.JSON.name(),targetUrl,paramMap,response);
                        }else { //缓存中不存在 查询数据库
                            ApiInterface apiInfo= apiInterfaceService.getApiInfo(request.getAppKey());
                            if (apiInfo == null){
                                throw new GatewayException(GatewayResultCode.REQUEST_INFO_UNEXIST);
                            }
                            rangRequestTarget(ApiInfoOutTypeEnum.JSON.name(),apiInfo.getTargetUrl(),paramMap,response);
                        }
                        break;
                    case "10004": //自定义查询(三方查询)
                        if (StringUtils.isNotBlank(apiInterface)){//redis中存在
                            JSONObject jsonCustom = JSONObject.parseObject(apiInterface);
                            String targetUrl = jsonCustom.getString("targetUrl");
                            rangRequestTarget(ApiInfoOutTypeEnum.JSON.name(),targetUrl,null,response);
                        }else {//redis中不存在,查询数据库
                            ApiInterface apiReqDto =
                                    apiInterfaceService.getReqTargetInfo(request.getAppKey());
                            if (null == apiReqDto) {
                                throw new GatewayException(GatewayResultCode.REQUEST_INFO_UNEXIST);
                            }
                            rangRequestTarget(ApiInfoOutTypeEnum.JSON.name(),apiReqDto.getTargetUrl(),null,response);
                        }
                        break;
                    case "10006": //数据银行查询
                        if (StringUtils.isNotBlank(apiInterface)) {//redis中存在
                            JSONObject jsonObject = JSONObject.parseObject(apiInterface);
                            String targetUrl = jsonObject.getString("targetUrl");
                            String outputType = jsonObject.getString("outputType");
                            rangRequestTarget(outputType, targetUrl, paramMap, response);
                        } else {//不存在查询数据库
                            ApiInterface apiReqDto =
                                    apiInterfaceService.getReqTargetInfo(request.getAppKey());
                            if (null == apiReqDto) {
                                throw new GatewayException(GatewayResultCode.REQUEST_INFO_UNEXIST);
                            }
                            rangRequestTarget(apiReqDto.getOutputType(), apiReqDto.getTargetUrl(),paramMap,response);
                        }
                        break;
                    case "10007": //数据银行数据包查询
                        if (StringUtils.isNotBlank(apiInterface)){//从redis中获取数据
                            JSONObject jsonObject = JSONObject.parseObject(apiInterface);
                            String targetUrl = jsonObject.getString("targetUrl");
                            String outputType = jsonObject.getString("outputType");
                            rangRequestTarget(outputType, targetUrl, paramMap, response);
                        }else {//不存在查询数据库
                            ApiInterface apiReqDto =
                                    apiInterfaceService.getReqTargetInfo(request.getAppKey());
                            if (null == apiReqDto) {
                                throw new GatewayException(GatewayResultCode.REQUEST_INFO_UNEXIST);
                            }
                            rangRequestTarget(apiReqDto.getOutputType(), apiReqDto.getTargetUrl(),paramMap,response);

                        }
                        break;
                    default:
                        response.setCode(GatewayResultCode.ILLEGAL_REQUEST.getCode());
                        response.setMsg(GatewayResultCode.ILLEGAL_REQUEST.getMsg());
                        break;
                }
            } catch (Exception e) {
                log.error("~~~~~~~~~~~~~~~请求api信息异常~~~~~~~~~~~~~");
                log.error("异常信息:{}", e.getMessage());
                response.setCode(GatewayResultCode.ILLEGAL_REQUEST.getCode());
                response.setMsg(GatewayResultCode.ILLEGAL_REQUEST.getMsg());
            }
        }
    }

    private void rangRequestTarget(String outputType, String targetUrl, Map<String, String> param, OpenApiResponse response) {
        if (ApiInfoOutTypeEnum.FLOW.name().equals(outputType)) {//文件流形式请求
            // httpsUtils.doGetImg()TODO 流形式未实现
        } else if (ApiInfoOutTypeEnum.JSON.name().equals(outputType)) {
            String respResult = httpsUtils.doGet(targetUrl, param);
            response.setAttribute(respResult);
        } else {
            throw new GatewayException(GatewayResultCode.OUTPUT_TYPE_EXCEPTION);
        }
    }
}
